@@ -54,7 +54,10 @@ girara_cmd_map_unmap(girara_session_t* session, girara_list_t* argument_list,
     {"ShiftTab",  GDK_KEY_ISO_Left_Tab},
     {"Up",        GDK_KEY_Up},
     {"Print",     GDK_KEY_Print},
-    {"Home",      GDK_KEY_Home}
+    {"KPLeft",    GDK_KEY_KP_Left},
+    {"KPRight",   GDK_KEY_KP_Right},
+    {"KPUp",      GDK_KEY_KP_Up},
+    {"KPDown",    GDK_KEY_KP_Down}
   };
 
   typedef struct gdk_mouse_button_s
@@ -133,13 +136,14 @@ girara_cmd_map_unmap(girara_session_t* session, girara_list_t* argument_list,
   if (tmp_length >= 3 && tmp[0] == '[' && tmp[tmp_size - 1] == ']') {
     char* tmp_inner = g_strndup(tmp + 1, tmp_size - 2);
 
-    GIRARA_LIST_FOREACH_BODY(session->modes.identifiers, girara_mode_string_t*, mode,
+    for (size_t idx = 0; idx != girara_list_size(session->modes.identifiers); ++idx) {
+      girara_mode_string_t* mode = girara_list_nth(session->modes.identifiers, idx);
       if (!g_strcmp0(tmp_inner, mode->name)) {
         shortcut_mode = mode->index;
         is_mode       = true;
         break;
       }
-    );
+    }
 
     if (is_mode == false) {
       girara_warning("Unregistered mode specified: %s", tmp_inner);
@@ -326,13 +330,14 @@ girara_cmd_map_unmap(girara_session_t* session, girara_list_t* argument_list,
   /* Check for passed shortcut command */
   if (unmap == false) {
     bool found_mapping = false;
-    GIRARA_LIST_FOREACH_BODY(session_private->config.shortcut_mappings, girara_shortcut_mapping_t*, mapping,
+    for (size_t idx = 0; idx != girara_list_size(session_private->config.shortcut_mappings); ++idx) {
+      girara_shortcut_mapping_t* mapping = girara_list_nth(session_private->config.shortcut_mappings, idx);
       if (!g_strcmp0(tmp, mapping->identifier)) {
         shortcut_function = mapping->function;
-        found_mapping = true;
+        found_mapping     = true;
         break;
       }
-    );
+    }
 
     if (found_mapping == false) {
       girara_warning("Not a valid shortcut function: %s", tmp);
@@ -349,12 +354,13 @@ girara_cmd_map_unmap(girara_session_t* session, girara_list_t* argument_list,
     if (++current_command < number_of_arguments) {
       tmp = (char*) girara_list_nth(argument_list, current_command);
 
-      GIRARA_LIST_FOREACH_BODY(session_private->config.argument_mappings, girara_argument_mapping_t*, mapping,
+      for (size_t idx = 0; idx != girara_list_size(session_private->config.argument_mappings); ++idx) {
+        girara_argument_mapping_t* mapping = girara_list_nth(session_private->config.argument_mappings, idx);
         if (!g_strcmp0(tmp, mapping->identifier)) {
           shortcut_argument_n = mapping->value;
           break;
         }
-      );
+      }
 
       /* If no known argument is passed we save it in the data field */
       if (shortcut_argument_n == 0) {
@@ -529,18 +535,16 @@ girara_cmd_set(girara_session_t* session, girara_list_t* argument_list)
   return true;
 }
 
-bool
-girara_inputbar_command_add(girara_session_t* session, const char* command,
-    const char* abbreviation, girara_command_function_t function,
-    girara_completion_function_t completion, const char* description)
-{
-  g_return_val_if_fail(session  != NULL, false);
-  g_return_val_if_fail(command  != NULL, false);
+bool girara_inputbar_command_add(girara_session_t* session, const char* command, const char* abbreviation,
+                                 girara_command_function_t function, girara_completion_function_t completion,
+                                 const char* description) {
+  g_return_val_if_fail(session != NULL, false);
+  g_return_val_if_fail(command != NULL, false);
   g_return_val_if_fail(function != NULL, false);
 
-  bool found = false;
   /* search for existing binding */
-  GIRARA_LIST_FOREACH_BODY(session->bindings.commands, girara_command_t*, commands_it,
+  for (size_t idx = 0; idx != girara_list_size(session->bindings.commands); ++idx) {
+    girara_command_t* commands_it = girara_list_nth(session->bindings.commands, idx);
     if (g_strcmp0(commands_it->command, command) == 0) {
       g_free(commands_it->abbr);
       g_free(commands_it->description);
@@ -550,81 +554,67 @@ girara_inputbar_command_add(girara_session_t* session, const char* command,
       commands_it->completion  = completion;
       commands_it->description = description ? g_strdup(description) : NULL;
 
-      found = true;
-      break;
+      return true;
     }
-  );
+  };
 
-  if (found == false) {
-    /* add new inputbar command */
-    girara_command_t* new_command = g_slice_new(girara_command_t);
+  /* add new inputbar command */
+  girara_command_t* new_command = g_malloc0(sizeof(girara_command_t));
 
-    new_command->command     = g_strdup(command);
-    new_command->abbr        = abbreviation ? g_strdup(abbreviation) : NULL;
-    new_command->function    = function;
-    new_command->completion  = completion;
-    new_command->description = description ? g_strdup(description) : NULL;
-    girara_list_append(session->bindings.commands, new_command);
-  }
+  new_command->command     = g_strdup(command);
+  new_command->abbr        = abbreviation ? g_strdup(abbreviation) : NULL;
+  new_command->function    = function;
+  new_command->completion  = completion;
+  new_command->description = description ? g_strdup(description) : NULL;
+  girara_list_append(session->bindings.commands, new_command);
 
   return true;
 }
 
-bool
-girara_special_command_add(girara_session_t* session, char identifier, girara_inputbar_special_function_t function, bool always, int argument_n, void* argument_data)
-{
-  g_return_val_if_fail(session  != NULL, false);
+bool girara_special_command_add(girara_session_t* session, char identifier, girara_inputbar_special_function_t function,
+                                bool always, int argument_n, void* argument_data) {
+  g_return_val_if_fail(session != NULL, false);
   g_return_val_if_fail(function != NULL, false);
 
   girara_argument_t argument = {argument_n, argument_data};
-  bool found                 = false;
 
   /* search for existing special command */
-  GIRARA_LIST_FOREACH_BODY_WITH_ITER(session->bindings.special_commands, girara_special_command_t*, iter, scommand_it,
+  for (size_t idx = 0; idx != girara_list_size(session->bindings.special_commands); ++idx) {
+    girara_special_command_t* scommand_it = girara_list_nth(session->bindings.special_commands, idx);
     if (scommand_it->identifier == identifier) {
       scommand_it->function = function;
       scommand_it->always   = always;
       scommand_it->argument = argument;
-      found = true;
-      break;
+
+      return true;
     }
-  );
+  };
 
-  if (found == false) {
-    /* create new special command */
-    girara_special_command_t* special_command = g_slice_new(girara_special_command_t);
+  /* create new special command */
+  girara_special_command_t* special_command = g_malloc0(sizeof(girara_special_command_t));
 
-    special_command->identifier = identifier;
-    special_command->function   = function;
-    special_command->always     = always;
-    special_command->argument   = argument;
-
-    girara_list_append(session->bindings.special_commands, special_command);
-  }
+  special_command->identifier = identifier;
+  special_command->function   = function;
+  special_command->always     = always;
+  special_command->argument   = argument;
+  girara_list_append(session->bindings.special_commands, special_command);
 
   return true;
 }
 
-void
-girara_special_command_free(girara_special_command_t* special_command)
-{
-  if (special_command == NULL) {
-    return;
-  }
-  g_slice_free(girara_special_command_t, special_command);
+void girara_special_command_free(girara_special_command_t* special_command) {
+  g_free(special_command);
 }
 
-void
-girara_command_free(girara_command_t* command)
-{
+void girara_command_free(girara_command_t* command) {
   if (command == NULL) {
     return;
   }
 
-  g_free(command->command);
-  g_free(command->abbr);
   g_free(command->description);
-  g_slice_free(girara_command_t, command);
+  g_free(command->abbr);
+  g_free(command->command);
+  g_free(command);
 }
 
 bool
@@ -652,21 +642,20 @@ girara_command_run(girara_session_t* session, const char* input)
   gchar *cmd = argv[0];
 
   /* search commands */
-  GIRARA_LIST_FOREACH_BODY_WITH_ITER(session->bindings.commands, girara_command_t*, iter, binding_command,
-    if ((g_strcmp0(cmd, binding_command->command) == 0) ||
-        (g_strcmp0(cmd, binding_command->abbr)    == 0)) {
+  for (size_t idx = 0; idx != girara_list_size(session->bindings.commands); ++idx) {
+    girara_command_t* binding_command = girara_list_nth(session->bindings.commands, idx);
+    if ((g_strcmp0(cmd, binding_command->command) == 0) || (g_strcmp0(cmd, binding_command->abbr) == 0)) {
       girara_list_t* argument_list = girara_list_new();
       if (argument_list == NULL) {
         g_strfreev(argv);
-        girara_list_iterator_free(iter);
         return false;
       }
 
       girara_list_set_free_function(argument_list, g_free);
 
-      for(int i = 1; i < argc; i++) {
+      for (int i = 1; i < argc; i++) {
         char* argument = g_strdup(argv[i]);
-        girara_list_append(argument_list, (void*) argument);
+        girara_list_append(argument_list, (void*)argument);
       }
 
       binding_command->function(session, argument_list);
@@ -680,10 +669,9 @@ girara_command_run(girara_session_t* session, const char* input)
         gtk_widget_hide(GTK_WIDGET(session->gtk.inputbar));
       }
       gtk_widget_hide(GTK_WIDGET(session->gtk.inputbar_dialog));
-      girara_list_iterator_free(iter);
       return true;
     }
-  );
+  }
 
   /* check for unknown command event handler */
   if (session->events.unknown_command != NULL) {
