@@ -94,10 +94,16 @@ css_template_fill_font(GiraraTemplate* csstemplate, const char* font)
   girara_template_set_variable_value(csstemplate, "font-family",
       pango_font_description_get_family(descr));
 
-  char* size = g_strdup_printf("%d%s", pango_font_description_get_size(descr) / PANGO_SCALE,
-      pango_font_description_get_size_is_absolute(descr) == FALSE ? "pt" : "");
-  girara_template_set_variable_value(csstemplate, "font-size", size);
-  g_free(size);
+  const int font_size = pango_font_description_get_size(descr);
+  if (!font_size) {
+    girara_debug("no font size given, defaulting to 9");
+    girara_template_set_variable_value(csstemplate, "font-size", "9pt");
+  } else {
+    char* size = g_strdup_printf("%d%s", pango_font_description_get_size(descr) / PANGO_SCALE,
+        pango_font_description_get_size_is_absolute(descr) == FALSE ? "pt" : "");
+    girara_template_set_variable_value(csstemplate, "font-size", size);
+    g_free(size);
+  }
 
   switch (pango_font_description_get_weight(descr)) {
     case PANGO_WEIGHT_THIN:
@@ -295,8 +301,8 @@ girara_session_create(void)
 {
   ensure_gettext_initialized();
 
-  girara_session_t* session = g_slice_alloc0(sizeof(girara_session_t));
-  session->private_data     = g_slice_alloc0(sizeof(girara_session_private_t));
+  girara_session_t* session = g_malloc0(sizeof(girara_session_t));
+  session->private_data     = g_malloc0(sizeof(girara_session_private_t));
 
   girara_session_private_t* session_private = session->private_data;
 
@@ -374,6 +380,8 @@ girara_session_create(void)
 
   /* make notification text selectable */
   gtk_label_set_selectable(GTK_LABEL(session->gtk.notification_text), TRUE);
+  /* ellipsize notification text */
+  gtk_label_set_ellipsize(GTK_LABEL(session->gtk.notification_text), PANGO_ELLIPSIZE_END);
 
   return session;
 }
@@ -533,11 +541,12 @@ girara_session_init(girara_session_t* session, const char* sessionname)
 
   /* box */
   gtk_container_add(GTK_CONTAINER(session->private_data->gtk.overlay), GTK_WIDGET(session->gtk.box));
-  /* overlay */
+  /* bottom_box */
   g_object_set(session->private_data->gtk.bottom_box, "halign", GTK_ALIGN_FILL, NULL);
   g_object_set(session->private_data->gtk.bottom_box, "valign", GTK_ALIGN_END, NULL);
 
-  gtk_overlay_add_overlay(GTK_OVERLAY(session->private_data->gtk.overlay), GTK_WIDGET(session->private_data->gtk.bottom_box));
+  gtk_container_add(GTK_CONTAINER(session->gtk.box), GTK_WIDGET(session->private_data->gtk.bottom_box));
+
   gtk_container_add(GTK_CONTAINER(session->gtk.window), GTK_WIDGET(session->private_data->gtk.overlay));
 
   /* statusbar */
@@ -629,7 +638,7 @@ girara_session_private_free(girara_session_private_t* session)
   /* clean up mutex */
   g_mutex_clear(&session->feedkeys_mutex);
 
-  g_slice_free(girara_session_private_t, session);
+  g_free(session);
 }
 
 bool
@@ -675,7 +684,7 @@ girara_session_destroy(girara_session_t* session)
   session->private_data = NULL;
 
   /* clean up session */
-  g_slice_free(girara_session_t, session);
+  g_free(session);
 
   return TRUE;
 }
@@ -853,16 +862,17 @@ girara_mode_add(girara_session_t* session, const char* name)
   g_return_val_if_fail(name != NULL && name[0] != '\0', FALSE);
 
   girara_mode_t last_index = 0;
-  GIRARA_LIST_FOREACH_BODY(session->modes.identifiers, girara_mode_string_t*, mode,
+  for (size_t idx = 0; idx != girara_list_size(session->modes.identifiers); ++idx) {
+    girara_mode_string_t* mode = girara_list_nth(session->modes.identifiers, idx);
     if (mode->index > last_index) {
       last_index = mode->index;
     }
-  );
+  }
 
   /* create new mode identifier */
-  girara_mode_string_t* mode = g_slice_new(girara_mode_string_t);
-  mode->index = last_index + 1;
-  mode->name = g_strdup(name);
+  girara_mode_string_t* mode = g_malloc(sizeof(girara_mode_string_t));
+  mode->index                = last_index + 1;
+  mode->name                 = g_strdup(name);
   girara_list_append(session->modes.identifiers, mode);
 
   return mode->index;
@@ -876,7 +886,7 @@ girara_mode_string_free(girara_mode_string_t* mode)
   }
 
   g_free(mode->name);
-  g_slice_free(girara_mode_string_t, mode);
+  g_free(mode);
 }
 
 girara_mode_t
