@@ -5,6 +5,7 @@
 #include "datastructures.h"
 #include "input-history.h"
 #include "internal.h"
+#include "log.h"
 #include "session.h"
 #include "settings.h"
 
@@ -12,46 +13,33 @@
 #include <string.h>
 
 static void girara_toggle_widget_visibility(GtkWidget* widget);
-static bool simulate_key_press(girara_session_t* session, int state, int key);
 
-bool
-girara_shortcut_add(girara_session_t* session, guint modifier, guint key, const char* buffer, girara_shortcut_function_t function, girara_mode_t mode, int argument_n, void* argument_data)
-{
+bool girara_shortcut_add(girara_session_t* session, guint modifier, guint key, const char* buffer,
+                         girara_shortcut_function_t function, girara_mode_t mode, int argument_n, void* argument_data) {
   g_return_val_if_fail(session != NULL, false);
   g_return_val_if_fail(buffer || key || modifier, false);
   g_return_val_if_fail(function != NULL, false);
 
-  girara_argument_t argument = {argument_n, (argument_data != NULL) ?
-    g_strdup(argument_data) : NULL};
+  girara_argument_t argument = {argument_n, (argument_data != NULL) ? g_strdup(argument_data) : NULL};
 
   /* search for existing binding */
-  bool found_existing_shortcut = false;
-  GIRARA_LIST_FOREACH_BODY_WITH_ITER(session->bindings.shortcuts, girara_shortcut_t*, iter, shortcuts_it,
+  for (size_t idx = 0; idx != girara_list_size(session->bindings.shortcuts); ++idx) {
+    girara_shortcut_t* shortcuts_it = girara_list_nth(session->bindings.shortcuts, idx);
     if (((shortcuts_it->mask == modifier && shortcuts_it->key == key && (modifier != 0 || key != 0)) ||
-       (buffer && shortcuts_it->buffered_command && !g_strcmp0(shortcuts_it->buffered_command, buffer)))
-        && ((shortcuts_it->mode == mode) || (mode == 0)))
-    {
+         (buffer && shortcuts_it->buffered_command && !g_strcmp0(shortcuts_it->buffered_command, buffer))) &&
+        ((shortcuts_it->mode == mode) || (mode == 0))) {
       if (shortcuts_it->argument.data != NULL) {
         g_free(shortcuts_it->argument.data);
       }
 
-      shortcuts_it->function  = function;
-      shortcuts_it->argument  = argument;
-      found_existing_shortcut = true;
-
-      if (mode != 0) {
-        girara_list_iterator_free(iter);
-        return true;
-      }
+      shortcuts_it->function = function;
+      shortcuts_it->argument = argument;
+      return true;
     }
-  );
-
-  if (found_existing_shortcut == true) {
-    return true;
   }
 
   /* add new shortcut */
-  girara_shortcut_t* shortcut = g_slice_new(girara_shortcut_t);
+  girara_shortcut_t* shortcut = g_malloc(sizeof(girara_shortcut_t));
 
   shortcut->mask             = modifier;
   shortcut->key              = key;
@@ -64,91 +52,80 @@ girara_shortcut_add(girara_session_t* session, guint modifier, guint key, const
   return true;
 }
 
-bool
-girara_shortcut_remove(girara_session_t* session, guint modifier, guint key, const char* buffer, girara_mode_t mode)
-{
+bool girara_shortcut_remove(girara_session_t* session, guint modifier, guint key, const char* buffer,
+                            girara_mode_t mode) {
   g_return_val_if_fail(session != NULL, false);
   g_return_val_if_fail(buffer || key || modifier, false);
 
-  bool handled = false;
   /* search for existing binding */
-  GIRARA_LIST_FOREACH_BODY(session->bindings.shortcuts, girara_shortcut_t*, shortcuts_it,
+  for (size_t idx = 0; idx != girara_list_size(session->bindings.shortcuts); ++idx) {
+    girara_shortcut_t* shortcuts_it = girara_list_nth(session->bindings.shortcuts, idx);
     if (((shortcuts_it->mask == modifier && shortcuts_it->key == key && (modifier != 0 || key != 0)) ||
-       (buffer && shortcuts_it->buffered_command && !g_strcmp0(shortcuts_it->buffered_command, buffer)))
-        && shortcuts_it->mode == mode) {
+         (buffer && shortcuts_it->buffered_command && !g_strcmp0(shortcuts_it->buffered_command, buffer))) &&
+        shortcuts_it->mode == mode) {
       girara_list_remove(session->bindings.shortcuts, shortcuts_it);
-      handled = true;
-      break;
+      return true;
     }
-  );
+  }
 
-  return handled;
+  return false;
 }
 
-void
-girara_shortcut_free(girara_shortcut_t* shortcut)
-{
+void girara_shortcut_free(girara_shortcut_t* shortcut) {
   g_return_if_fail(shortcut != NULL);
   g_free(shortcut->buffered_command);
   g_free(shortcut->argument.data);
-  g_slice_free(girara_shortcut_t, shortcut);
+  g_free(shortcut);
 }
 
-bool
-girara_inputbar_shortcut_add(girara_session_t* session, guint modifier, guint key, girara_shortcut_function_t function, int argument_n, void* argument_data)
-{
-  g_return_val_if_fail(session  != NULL, false);
+bool girara_inputbar_shortcut_add(girara_session_t* session, guint modifier, guint key,
+                                  girara_shortcut_function_t function, int argument_n, void* argument_data) {
+  g_return_val_if_fail(session != NULL, false);
   g_return_val_if_fail(function != NULL, false);
 
   girara_argument_t argument = {argument_n, argument_data};
-  bool found = false;
 
   /* search for existing special command */
-  GIRARA_LIST_FOREACH_BODY(session->bindings.inputbar_shortcuts, girara_inputbar_shortcut_t*, inp_sh_it,
+  for (size_t idx = 0; idx != girara_list_size(session->bindings.inputbar_shortcuts); ++idx) {
+    girara_inputbar_shortcut_t* inp_sh_it = girara_list_nth(session->bindings.inputbar_shortcuts, idx);
     if (inp_sh_it->mask == modifier && inp_sh_it->key == key) {
       inp_sh_it->function = function;
       inp_sh_it->argument = argument;
 
-      found = true;
+      return true;
+    }
+  }
+
+  /* create new inputbar shortcut */
+  girara_inputbar_shortcut_t* inputbar_shortcut = g_malloc(sizeof(girara_inputbar_shortcut_t));
+
+  inputbar_shortcut->mask     = modifier;
+  inputbar_shortcut->key      = key;
+  inputbar_shortcut->function = function;
+  inputbar_shortcut->argument = argument;
+
+  girara_list_append(session->bindings.inputbar_shortcuts, inputbar_shortcut);
+
+  return true;
+}
+
+bool girara_inputbar_shortcut_remove(girara_session_t* session, guint modifier, guint key) {
+  g_return_val_if_fail(session != NULL, false);
+
+  /* search for existing special command */
+  for (size_t idx = 0; idx != girara_list_size(session->bindings.inputbar_shortcuts); ++idx) {
+    girara_inputbar_shortcut_t* inp_sh_it = girara_list_nth(session->bindings.inputbar_shortcuts, idx);
+    if (inp_sh_it->mask == modifier && inp_sh_it->key == key) {
+      girara_list_remove(session->bindings.inputbar_shortcuts, inp_sh_it);
       break;
     }
-  );
-
-  if (found == false) {
-    /* create new inputbar shortcut */
-    girara_inputbar_shortcut_t* inputbar_shortcut = g_slice_new(girara_inputbar_shortcut_t);
-
-    inputbar_shortcut->mask     = modifier;
-    inputbar_shortcut->key      = key;
-    inputbar_shortcut->function = function;
-    inputbar_shortcut->argument = argument;
-
-    girara_list_append(session->bindings.inputbar_shortcuts, inputbar_shortcut);
   }
 
   return true;
 }
 
-bool
-girara_inputbar_shortcut_remove(girara_session_t* session, guint modifier, guint key)
-{
-  g_return_val_if_fail(session  != NULL, false);
-
-  /* search for existing special command */
-  GIRARA_LIST_FOREACH_BODY(session->bindings.inputbar_shortcuts, girara_inputbar_shortcut_t*, inp_sh_it,
-    if (inp_sh_it->mask == modifier && inp_sh_it->key == key) {
-      girara_list_remove(session->bindings.inputbar_shortcuts, inp_sh_it);
-      break;
-    }
-  );
-
-  return true;
-}
-
-void
-girara_inputbar_shortcut_free(girara_inputbar_shortcut_t* inputbar_shortcut)
-{
-  g_slice_free(girara_inputbar_shortcut_t, inputbar_shortcut);
+void girara_inputbar_shortcut_free(girara_inputbar_shortcut_t* inputbar_shortcut) {
+  g_free(inputbar_shortcut);
 }
 
 bool
@@ -460,6 +437,64 @@ girara_sc_exec(girara_session_t* session, girara_argument_t* argument, girara_ev
   return false;
 }
 
+static bool
+simulate_key_press(girara_session_t* session, int state, int key)
+{
+  if (session == NULL || session->gtk.box == NULL) {
+    return false;
+  }
+
+  GdkEvent* event = gdk_event_new(GDK_KEY_PRESS);
+
+  event->any.type       = GDK_KEY_PRESS;
+  event->key.window     = g_object_ref(gtk_widget_get_parent_window(GTK_WIDGET(session->gtk.box)));
+  event->key.send_event = false;
+  event->key.time       = GDK_CURRENT_TIME;
+  event->key.state      = state;
+  event->key.keyval     = key;
+
+  GdkDisplay* display = gtk_widget_get_display(GTK_WIDGET(session->gtk.box));
+  GdkKeymapKey* keys  = NULL;
+  gint number_of_keys = 0;
+
+  if (gdk_keymap_get_entries_for_keyval(gdk_keymap_get_for_display(display),
+        event->key.keyval, &keys, &number_of_keys) == FALSE) {
+    gdk_event_free(event);
+    return false;
+  }
+
+  event->key.hardware_keycode = keys[0].keycode;
+  event->key.group            = keys[0].group;
+
+  g_free(keys);
+
+  GdkSeat* seat = gdk_display_get_default_seat(display);
+  GdkDevice* keyboard = gdk_seat_get_keyboard(seat);
+  gdk_event_set_device(event, keyboard);
+
+  gdk_event_put(event);
+  gdk_event_free(event);
+
+  gtk_main_iteration_do(FALSE);
+
+  return true;
+}
+
+static int
+update_state_by_keyval(int state, int keyval)
+{
+  /* The following is probably not true for some keyboard layouts. */
+  if ((keyval >= '!' && keyval <= '/')
+      || (keyval >= ':' && keyval <= '@')
+      || (keyval >= '[' && keyval <= '`')
+      || (keyval >= '{' && keyval <= '~')
+      ) {
+    state |= GDK_SHIFT_MASK;
+  }
+
+  return state;
+}
+
 bool
 girara_sc_feedkeys(girara_session_t* session, girara_argument_t* argument,
     girara_event_t* UNUSED(event), unsigned int t)
@@ -468,6 +503,11 @@ girara_sc_feedkeys(girara_session_t* session, girara_argument_t* argument,
     return false;
   }
 
+  if (g_mutex_trylock(&session->private_data->feedkeys_mutex) == FALSE) {
+    girara_error("Recursive use of feedkeys detected. Aborting evaluation.");
+    return false;
+  }
+
   typedef struct gdk_keyboard_button_s {
     char* identifier;
     int keyval;
@@ -501,13 +541,18 @@ girara_sc_feedkeys(girara_session_t* session, girara_argument_t* argument,
     {"Super",     GDK_KEY_Super_L},
     {"Tab",       GDK_KEY_Tab},
     {"ShiftTab",  GDK_KEY_ISO_Left_Tab},
-    {"Up",        GDK_KEY_Up}
+    {"Up",        GDK_KEY_Up},
+    {"Print",     GDK_KEY_Print},
+    {"KPLeft",    GDK_KEY_KP_Left},
+    {"KPRight",   GDK_KEY_KP_Right},
+    {"KPUp",      GDK_KEY_KP_Up},
+    {"KPDown",    GDK_KEY_KP_Down}
   };
 
   char* input               = (char*) argument->data;
   unsigned int input_length = strlen(input);
 
-  t = (t == 0) ? 1 : t;
+  t = MAX(1, t);
   for (unsigned int c = 0; c < t; c++) {
     for (unsigned i = 0; i < input_length; i++) {
       int state  = 0;
@@ -520,7 +565,7 @@ girara_sc_feedkeys(girara_session_t* session, girara_argument_t* argument,
           goto single_key;
         }
 
-        int length = end - (input + i) - 1;
+        const int length = end - (input + i) - 1;
         char* tmp  = g_strndup(input + i + 1, length);
         bool found = false;
 
@@ -572,203 +617,132 @@ girara_sc_feedkeys(girara_session_t* session, girara_argument_t* argument,
       }
 
 single_key:
-
-      update_state_by_keyval(&state, keyval);
+      state = update_state_by_keyval(state, keyval);
       simulate_key_press(session, state, keyval);
     }
   }
 
+  g_mutex_unlock(&session->private_data->feedkeys_mutex);
   return true;
 }
 
-bool
-girara_shortcut_mapping_add(girara_session_t* session, const char* identifier, girara_shortcut_function_t function)
-{
-  g_return_val_if_fail(session  != NULL, false);
+bool girara_shortcut_mapping_add(girara_session_t* session, const char* identifier,
+                                 girara_shortcut_function_t function) {
+  g_return_val_if_fail(session != NULL, false);
 
   if (function == NULL || identifier == NULL) {
     return false;
   }
 
   girara_session_private_t* session_private = session->private_data;
-  bool found = false;
-
-  GIRARA_LIST_FOREACH_BODY(session_private->config.shortcut_mappings, girara_shortcut_mapping_t*, data,
+  for (size_t idx = 0; idx != girara_list_size(session_private->config.shortcut_mappings); ++idx) {
+    girara_shortcut_mapping_t* data = girara_list_nth(session_private->config.shortcut_mappings, idx);
     if (g_strcmp0(data->identifier, identifier) == 0) {
       data->function = function;
-      found = true;
-      break;
+      return true;
     }
-  );
-
-  if (found == false) {
-    /* add new config handle */
-    girara_shortcut_mapping_t* mapping = g_slice_new(girara_shortcut_mapping_t);
-
-    mapping->identifier = g_strdup(identifier);
-    mapping->function   = function;
-    girara_list_append(session_private->config.shortcut_mappings, mapping);
   }
 
+  /* add new config handle */
+  girara_shortcut_mapping_t* mapping = g_malloc(sizeof(girara_shortcut_mapping_t));
+
+  mapping->identifier = g_strdup(identifier);
+  mapping->function   = function;
+  girara_list_append(session_private->config.shortcut_mappings, mapping);
+
   return true;
 }
 
-void
-girara_shortcut_mapping_free(girara_shortcut_mapping_t* mapping)
-{
+void girara_shortcut_mapping_free(girara_shortcut_mapping_t* mapping) {
   if (mapping == NULL) {
     return;
   }
 
   g_free(mapping->identifier);
-  g_slice_free(girara_shortcut_mapping_t, mapping);
+  g_free(mapping);
 }
 
-bool
-girara_argument_mapping_add(girara_session_t* session, const char* identifier, int value)
-{
-  g_return_val_if_fail(session  != NULL, false);
+bool girara_argument_mapping_add(girara_session_t* session, const char* identifier, int value) {
+  g_return_val_if_fail(session != NULL, false);
 
   if (identifier == NULL) {
     return false;
   }
 
   girara_session_private_t* session_private = session->private_data;
-  bool found = false;
-
-  GIRARA_LIST_FOREACH_BODY(session_private->config.argument_mappings, girara_argument_mapping_t*, mapping,
+  for (size_t idx = 0; idx != girara_list_size(session_private->config.argument_mappings); ++idx) {
+    girara_argument_mapping_t* mapping = girara_list_nth(session_private->config.argument_mappings, idx);
     if (g_strcmp0(mapping->identifier, identifier) == 0) {
       mapping->value = value;
-      found = true;
-      break;
+      return true;
     }
-  );
-
-  if (found == false) {
-    /* add new config handle */
-    girara_argument_mapping_t* mapping = g_slice_new(girara_argument_mapping_t);
-
-    mapping->identifier = g_strdup(identifier);
-    mapping->value      = value;
-    girara_list_append(session_private->config.argument_mappings, mapping);
   }
 
+  /* add new config handle */
+  girara_argument_mapping_t* mapping = g_malloc(sizeof(girara_argument_mapping_t));
+
+  mapping->identifier = g_strdup(identifier);
+  mapping->value      = value;
+  girara_list_append(session_private->config.argument_mappings, mapping);
+
   return true;
 }
 
-void
-girara_argument_mapping_free(girara_argument_mapping_t* argument_mapping)
-{
+void girara_argument_mapping_free(girara_argument_mapping_t* argument_mapping) {
   if (argument_mapping == NULL) {
     return;
   }
 
   g_free(argument_mapping->identifier);
-  g_slice_free(girara_argument_mapping_t, argument_mapping);
+  g_free(argument_mapping);
 }
 
-bool
-girara_mouse_event_add(girara_session_t* session, guint mask, guint button,
-    girara_shortcut_function_t function, girara_mode_t mode, girara_event_type_t
-    event_type, int argument_n, void* argument_data)
-{
-  g_return_val_if_fail(session  != NULL, false);
+bool girara_mouse_event_add(girara_session_t* session, guint mask, guint button, girara_shortcut_function_t function,
+                            girara_mode_t mode, girara_event_type_t event_type, int argument_n, void* argument_data) {
+  g_return_val_if_fail(session != NULL, false);
   g_return_val_if_fail(function != NULL, false);
 
   girara_argument_t argument = {argument_n, argument_data};
-  bool found = false;
 
   /* search for existing binding */
-  GIRARA_LIST_FOREACH_BODY(session->bindings.mouse_events, girara_mouse_event_t*, me_it,
-    if (me_it->mask == mask && me_it->button == button &&
-       me_it->mode == mode && me_it->event_type == event_type) {
+  for (size_t idx = 0; idx != girara_list_size(session->bindings.mouse_events); ++idx) {
+    girara_mouse_event_t* me_it = girara_list_nth(session->bindings.mouse_events, idx);
+    if (me_it->mask == mask && me_it->button == button && me_it->mode == mode && me_it->event_type == event_type) {
       me_it->function = function;
       me_it->argument = argument;
-
-      found = true;
-      break;
+      return true;
     }
-  );
-
-  if (found == false) {
-    /* add new mouse event */
-    girara_mouse_event_t* mouse_event = g_slice_new(girara_mouse_event_t);
-
-    mouse_event->mask       = mask;
-    mouse_event->button     = button;
-    mouse_event->function   = function;
-    mouse_event->mode       = mode;
-    mouse_event->event_type = event_type;
-    mouse_event->argument   = argument;
-    girara_list_append(session->bindings.mouse_events, mouse_event);
   }
 
+  /* add new mouse event */
+  girara_mouse_event_t* mouse_event = g_malloc(sizeof(girara_mouse_event_t));
+
+  mouse_event->mask       = mask;
+  mouse_event->button     = button;
+  mouse_event->function   = function;
+  mouse_event->mode       = mode;
+  mouse_event->event_type = event_type;
+  mouse_event->argument   = argument;
+  girara_list_append(session->bindings.mouse_events, mouse_event);
+
   return true;
 }
 
-bool
-girara_mouse_event_remove(girara_session_t* session, guint mask, guint button, girara_mode_t mode)
-{
-  g_return_val_if_fail(session  != NULL, false);
+bool girara_mouse_event_remove(girara_session_t* session, guint mask, guint button, girara_mode_t mode) {
+  g_return_val_if_fail(session != NULL, false);
 
-  bool found = false;
   /* search for existing binding */
-  GIRARA_LIST_FOREACH_BODY(session->bindings.mouse_events, girara_mouse_event_t*, me_it,
-    if (me_it->mask == mask && me_it->button == button &&
-       me_it->mode == mode) {
+  for (size_t idx = 0; idx != girara_list_size(session->bindings.mouse_events); ++idx) {
+    girara_mouse_event_t* me_it = girara_list_nth(session->bindings.mouse_events, idx);
+    if (me_it->mask == mask && me_it->button == button && me_it->mode == mode) {
       girara_list_remove(session->bindings.mouse_events, me_it);
-      found = true;
-      break;
+      return true;
     }
-  );
-
-  return found;
-}
-
-void
-girara_mouse_event_free(girara_mouse_event_t* mouse_event)
-{
-  if (mouse_event == NULL) {
-    return;
-  }
-  g_slice_free(girara_mouse_event_t, mouse_event);
-}
-
-static bool
-simulate_key_press(girara_session_t* session, int state, int key)
-{
-  if (session == NULL || session->gtk.box == NULL) {
-    return false;
   }
 
-  GdkEvent* event = gdk_event_new(GDK_KEY_PRESS);
+  return false;
+}
 
-  event->any.type       = GDK_KEY_PRESS;
-  event->key.window     = g_object_ref(gtk_widget_get_parent_window(GTK_WIDGET(session->gtk.box)));
-  event->key.send_event = false;
-  event->key.time       = GDK_CURRENT_TIME;
-  event->key.state      = state;
-  event->key.keyval     = key;
-
-  GdkDisplay* display = gtk_widget_get_display(GTK_WIDGET(session->gtk.box));
-  GdkKeymapKey* keys  = NULL;
-  gint number_of_keys = 0;
-
-  if (gdk_keymap_get_entries_for_keyval(gdk_keymap_get_for_display(display),
-        event->key.keyval, &keys, &number_of_keys) == FALSE) {
-    gdk_event_free(event);
-    return false;
-  }
-
-  event->key.hardware_keycode = keys[0].keycode;
-  event->key.group            = keys[0].group;
-
-  g_free(keys);
-
-  gdk_event_put(event);
-  gdk_event_free(event);
-
-  gtk_main_iteration_do(FALSE);
-
-  return true;
+void girara_mouse_event_free(girara_mouse_event_t* mouse_event) {
+  g_free(mouse_event);
 }
